@@ -1,8 +1,10 @@
 from copy import deepcopy
 from typing import Any, Union
 
+from .config import s3_config
+
 try:
-    from argo.workflows.client import V1alpha1S3Artifact
+    from argo.workflows.client import V1alpha1OSSArtifact, V1alpha1S3Artifact
     from argo.workflows.client.configuration import Configuration
 except Exception:
     V1alpha1S3Artifact = object
@@ -39,6 +41,12 @@ class S3Artifact(V1alpha1S3Artifact):
         artifact.key += path
         return artifact
 
+    def oss(self):
+        config = Configuration()
+        config.client_side_validation = False
+        return V1alpha1OSSArtifact(key=s3_config["repo_prefix"] + self.key,
+                                   local_vars_configuration=config)
+
 
 class LocalArtifact:
     def __init__(self, local_path):
