@@ -1,5 +1,6 @@
 config = {
     "host": "https://127.0.0.1:2746",
+    "namespace": "argo",
     "token": None,
     "k8s_config_file": None,
     "k8s_api_server": None,
@@ -26,6 +27,7 @@ def set_config(
 
     Args:
         host: host of Argo server
+        namespace: k8s namespace
         token: token for authentication, necessary for reused workflows
         k8s_config_file: location of kube config file if it is used for
         authentication
@@ -45,3 +47,42 @@ def set_config(
         mode: "default" for normal, "debug" for debugging locally
     """
     config.update(kwargs)
+
+
+s3_config = {
+    "endpoint": "127.0.0.1:9000",
+    "console": "http://127.0.0.1:9001",
+    "access_key": "admin",
+    "secret_key": "password",
+    "secure": False,
+    "bucket_name": "my-bucket",
+    "repo_key": None,
+    "repo": None,
+    "repo_type": "s3",
+    "repo_prefix": "",
+    "prefix": "",
+    "storage_client": None,
+}
+
+
+def set_s3_config(
+    **kwargs,
+) -> None:
+    """
+    Set S3 configurations
+
+    Args:
+        endpoint: endpoint for S3 storage
+        console: console address for S3 storage
+        access_key: access key for S3 storage
+        secret_key: secret key for S3 storage
+        secure: secure or not
+        bucket_name: name of S3 bucket
+        repo_key: key of artifact repository
+        repo: artifact repository, parsed from repo_key
+        repo_type: s3 or oss, parsed from repo_key
+        repo_prefix: prefix of artifact repository, parsed from repo_key
+        prefix: prefix of storage key
+        storage_client: client for plugin storage backend
+    """
+    s3_config.update(kwargs)
