@@ -8,7 +8,7 @@ import jsonpickle
 
 from .common import S3Artifact
 from .config import config
-from .utils import randstr, upload_s3
+from .utils import randstr, s3_config, upload_s3
 
 try:
     from argo.workflows.client import (V1alpha1ArchiveStrategy, V1alpha1Inputs,
@@ -346,8 +346,14 @@ class InputParameter(ArgoVar):
                             f.write(jsonpickle.dumps(content))
                         key = upload_s3(path)
                         s3 = S3Artifact(key=key)
-                    return V1alpha1Artifact(name="dflow_bigpar_" + self.name,
-                                            path=self.path, s3=s3)
+                    if s3_config["repo_type"] == "s3":
+                        return V1alpha1Artifact(
+                            name="dflow_bigpar_" + self.name, path=self.path,
+                            s3=s3)
+                    elif s3_config["repo_type"] == "oss":
+                        return V1alpha1Artifact(
+                            name="dflow_bigpar_" + self.name, path=self.path,
+                            oss=s3.oss())
             elif isinstance(self.source, (InputParameter, OutputParameter,
                                           InputArtifact, OutputArtifact)):
                 return V1alpha1Artifact(name="dflow_bigpar_" + self.name,
@@ -460,9 +466,15 @@ class InputArtifact(ArgoVar):
                                     _from=str(self.source), sub_path=sub_path,
                                     mode=self.mode)
         elif isinstance(self.source, S3Artifact):
-            return V1alpha1Artifact(name=self.name, path=self.path,
-                                    optional=self.optional, s3=self.source,
-                                    sub_path=self.sub_path, mode=self.mode)
+            if s3_config["repo_type"] == "s3":
+                return V1alpha1Artifact(name=self.name, path=self.path,
+                                        optional=self.optional, s3=self.source,
+                                        sub_path=self.sub_path, mode=self.mode)
+            else:
+                return V1alpha1Artifact(name=self.name, path=self.path,
+                                        optional=self.optional,
+                                        oss=self.source.oss(),
+                                        sub_path=self.sub_path, mode=self.mode)
         elif isinstance(self.source, str):
             return V1alpha1Artifact(name=self.name, path=self.path,
                                     optional=self.optional,
@@ -610,24 +622,19 @@ class OutputParameter(ArgoVar):
             description = jsonpickle.dumps({"type": str(self.type)})
 
         if self.save_as_artifact:
+            kwargs = {
+                "name": "dflow_bigpar_" + self.name,
+                "archive": V1alpha1ArchiveStrategy(_none={}),
+                "global_name": self.global_name,
+            }
             if self.value_from_path is not None:
-                return V1alpha1Artifact(
-                    name="dflow_bigpar_" + self.name,
-                    path=self.value_from_path,
-                    archive=V1alpha1ArchiveStrategy(_none={}),
-                    global_name=self.global_name)
+                return V1alpha1Artifact(path=self.value_from_path, **kwargs)
             elif self.value_from_parameter is not None:
                 return V1alpha1Artifact(
-                    name="dflow_bigpar_" + self.name,
-                    _from=str(self.value_from_parameter),
-                    archive=V1alpha1ArchiveStrategy(_none={}),
-                    global_name=self.global_name)
+                    _from=str(self.value_from_parameter), **kwargs)
             elif self.value_from_expression is not None:
                 return V1alpha1Artifact(
-                    name="dflow_bigpar_" + self.name,
-                    from_expression=str(self.value_from_expression),
-                    archive=V1alpha1ArchiveStrategy(_none={}),
-                    global_name=self.global_name)
+                    from_expression=str(self.value_from_expression), **kwargs)
             else:
                 raise RuntimeError("Not supported.")
 
@@ -797,10 +804,15 @@ class OutputArtifact(ArgoVar):
         return pvc
 
     def convert_to_argo(self):
+        kwargs = {
+            "name": self.name,
+            "global_name": self.global_name
+        }
+
         if self.archive is None:
-            archive = V1alpha1ArchiveStrategy(_none={})
+            kwargs["archive"] = V1alpha1ArchiveStrategy(_none={})
         elif self.archive == "tar":
-            archive = None
+            kwargs["archive"] = None
         else:
             raise RuntimeError("Archive type %s not supported" % self.archive)
 
@@ -809,18 +821,19 @@ class OutputArtifact(ArgoVar):
             if isinstance(save, S3Artifact):
                 s3 = save
 
+        if s3 is not None:
+            if s3_config["repo_type"] == "s3":
+                kwargs["s3"] = s3
+            elif s3_config["repo_type"] == "oss":
+                kwargs["oss"] = s3.oss()
+
         if self.path is not None:
-            return V1alpha1Artifact(name=self.name, path=self.path,
-                                    archive=archive, s3=s3,
-                                    global_name=self.global_name)
+            return V1alpha1Artifact(path=self.path, **kwargs)
         elif self._from is not None:
-            return V1alpha1Artifact(name=self.name, _from=str(self._from),
-                                    archive=archive, s3=s3,
-                                    global_name=self.global_name)
+            return V1alpha1Artifact(_from=str(self._from), **kwargs)
         elif self.from_expression is not None:
             return V1alpha1Artifact(
-                name=self.name, from_expression=str(self.from_expression),
-                archive=archive, s3=s3, global_name=self.global_name)
+                from_expression=str(self.from_expression), **kwargs)
         else:
             raise RuntimeError("Output artifact %s is not specified" % self)
 
