@@ -11,12 +11,14 @@ from .common import LocalArtifact, S3Artifact
 from .config import config
 from .context_syntax import GLOBAL_CONTEXT
 from .executor import Executor
-from .io import (PVC, ArgoVar, InputArtifact, InputParameter, OutputArtifact,
-                 OutputParameter)
+from .io import (PVC, ArgoVar, IfExpression, InputArtifact, InputParameter,
+                 OutputArtifact, OutputParameter)
 from .op_template import OPTemplate, PythonScriptOPTemplate, ShellOPTemplate
+from .python import Slices
 from .resource import Resource
 from .util_ops import CheckNumSuccess, CheckSuccessRatio, InitArtifactForSlices
-from .utils import catalog_of_artifact, randstr, upload_artifact
+from .utils import (catalog_of_artifact, merge_dir, randstr, s3_config,
+                    upload_artifact)
 
 try:
     from argo.workflows.client import (V1alpha1Arguments, V1alpha1ContinueOn,
@@ -40,6 +42,9 @@ class FutureLen:
     def __init__(self, par):
         self.par = par
 
+    def __repr__(self):
+        return "len(%s)" % self.par
+
     def get(self):
         return len(self.par.value)
 
@@ -48,13 +53,19 @@ class FutureRange:
     def __init__(self, *args):
         self.args = args
 
-    def get(self):
+    def get(self, context=None):
         args = []
         for i in self.args:
             if isinstance(i, FutureLen):
                 args.append(i.get())
             elif isinstance(i, (InputParameter, OutputParameter)):
                 args.append(i.value)
+            elif isinstance(i, IfExpression):
+                _if = render_expr(i._if, context)
+                if eval_bool_expr(_if):
+                    args.append(int(eval(render_expr(i._then, context))))
+                else:
+                    args.append(int(eval(render_expr(i._else, context))))
             else:
                 args.append(i)
         args = tuple(args)
@@ -178,6 +189,7 @@ class Step:
         util_image_pull_policy: image pull policy for utility step
         util_command: command for utility step
         parallelism: parallelism for sliced step
+        slices: override slices of OP template
     """
 
     def __init__(
@@ -201,6 +213,7 @@ class Step:
             util_image_pull_policy: str = None,
             util_command: Union[str, List[str]] = None,
             parallelism: int = None,
+            slices: Slices = None,
             **kwargs,
     ) -> None:
         self.name = name
@@ -259,6 +272,111 @@ class Step:
 
         new_template = None
 
+        if slices is not None:
+            new_template = deepcopy(self.template)
+            new_template.name = self.template.name + "-" + randstr()
+            new_template.slices = slices
+            for name, par in new_template.inputs.parameters.items():
+                if name not in self.inputs.parameters:
+                    self.inputs.parameters[name] = deepcopy(par)
+
+            new_template.inputs.parameters["dflow_item"] = InputParameter(
+                value=slices.slices)
+            self.inputs.parameters["dflow_item"] = InputParameter(
+                value=slices.slices)
+            for name in slices.input_parameter:
+                for step in (new_template if hasattr(new_template, "__iter__")
+                             else []):
+                    for par in list(step.inputs.parameters.values()):
+                        # input parameter referring to sliced input parameter
+                        if par.value is new_template.inputs.parameters[name]:
+                            step.template.inputs.parameters["dflow_item"] = \
+                                InputParameter()
+                            step.template.add_slices(Slices(
+                                "{{inputs.parameters.dflow_item}}",
+                                input_parameter=[par.name],
+                                sub_path=slices.sub_path,
+                                pool_size=slices.pool_size))
+                            step.template.render_script()
+                            step.inputs.parameters["dflow_item"] = \
+                                InputParameter(
+                                    value="{{inputs.parameters.dflow_item}}")
+
+            for name in slices.input_artifact:
+                for step in (new_template if hasattr(new_template, "__iter__")
+                             else []):
+                    for art in list(step.inputs.artifacts.values()):
+                        # input artifact referring to sliced input artifact
+                        if art.source is new_template.inputs.artifacts[name]:
+                            step.template.inputs.parameters["dflow_item"] = \
+                                InputParameter()
+                            step.template.add_slices(Slices(
+                                "{{inputs.parameters.dflow_item}}",
+                                input_artifact=[art.name],
+                                sub_path=slices.sub_path,
+                                pool_size=slices.pool_size))
+                            step.template.render_script()
+                            step.inputs.parameters["dflow_item"] = \
+                                InputParameter(
+                                    value="{{inputs.parameters.dflow_item}}")
+
+            def stack_output_parameter(par):
+                if isinstance(par, OutputParameter):
+                    step = par.step
+                    step.template.inputs.parameters["dflow_item"] = \
+                        InputParameter()
+                    step.template.add_slices(Slices(
+                        "{{inputs.parameters.dflow_item}}",
+                        output_parameter=[par.name],
+                        sub_path=slices.sub_path,
+                        pool_size=slices.pool_size))
+                    step.template.render_script()
+                    step.inputs.parameters["dflow_item"] = \
+                        InputParameter(
+                            value="{{inputs.parameters.dflow_item}}")
+
+            for name in slices.output_parameter:
+                # sliced output parameter from
+                if new_template.outputs.parameters[name].value_from_parameter\
+                        is not None:
+                    stack_output_parameter(new_template.outputs.parameters[
+                        name].value_from_parameter)
+                elif new_template.outputs.parameters[name].\
+                        value_from_expression is not None:
+                    stack_output_parameter(new_template.outputs.parameters[
+                        name].value_from_expression._then)
+                    stack_output_parameter(new_template.outputs.parameters[
+                        name].value_from_expression._else)
+
+            def stack_output_artifact(art):
+                if isinstance(art, OutputArtifact):
+                    step = art.step
+                    step.template.inputs.parameters["dflow_item"] = \
+                        InputParameter()
+                    step.template.add_slices(Slices(
+                        "{{inputs.parameters.dflow_item}}",
+                        output_artifact=[art.name],
+                        sub_path=slices.sub_path,
+                        pool_size=slices.pool_size))
+                    step.template.render_script()
+                    step.inputs.parameters["dflow_item"] = \
+                        InputParameter(
+                            value="{{inputs.parameters.dflow_item}}")
+
+            for name in slices.output_artifact:
+                # sliced output artifact from
+                if new_template.outputs.artifacts[name]._from is not None:
+                    stack_output_artifact(
+                        new_template.outputs.artifacts[name]._from)
+                elif new_template.outputs.artifacts[name].from_expression is \
+                        not None:
+                    stack_output_artifact(new_template.outputs.artifacts[name].
+                                          from_expression._then)
+                    stack_output_artifact(new_template.outputs.artifacts[name].
+                                          from_expression._else)
+
+            self.template = new_template
+
         if hasattr(self.template, "slices") and self.template.slices is not \
                 None and (self.template.slices.output_artifact or (
                     self.template.slices.sub_path and
@@ -281,8 +399,34 @@ class Step:
                 # artifacts are reused
                 for name in new_template.slices.output_artifact:
                     new_template.outputs.artifacts[name].save.append(
-                        S3Artifact(key="{{workflow.name}}/{{inputs."
-                                   "parameters.dflow_group_key}}/%s" % name))
+                        S3Artifact(key="%s{{workflow.name}}/{{inputs."
+                                   "parameters.dflow_group_key}}/%s" % (
+                                       s3_config["prefix"], name)))
+
+                    def merge_output_artifact(art):
+                        step = art.step
+                        template = step.template
+                        template.inputs.parameters["dflow_group_key"] = \
+                            InputParameter()
+                        step.inputs.parameters["dflow_group_key"] = \
+                            InputParameter(
+                                value="{{inputs.parameters.dflow_group_key}}")
+                        template.outputs.artifacts[art.name].save.append(
+                            S3Artifact(
+                                key="%s{{workflow.name}}/{{inputs."
+                                "parameters.dflow_group_key}}/%s" % (
+                                    s3_config["prefix"], name)))
+
+                    if new_template.outputs.artifacts[name]._from is not \
+                            None:
+                        merge_output_artifact(
+                            new_template.outputs.artifacts[name]._from)
+                    elif new_template.outputs.artifacts[name].\
+                            from_expression is not None:
+                        merge_output_artifact(new_template.outputs.artifacts[
+                            name].from_expression._then)
+                        merge_output_artifact(new_template.outputs.artifacts[
+                            name].from_expression._else)
             else:
                 new_template.inputs.parameters["dflow_artifact_key"] = \
                     InputParameter(value="")
@@ -291,6 +435,30 @@ class Step:
                         S3Artifact(key="{{inputs.parameters."
                                    "dflow_artifact_key}}/%s" % name))
 
+                    def merge_output_artifact(art):
+                        step = art.step
+                        template = step.template
+                        template.inputs.parameters["dflow_artifact_key"] = \
+                            InputParameter()
+                        step.inputs.parameters["dflow_artifact_key"] = \
+                            InputParameter(value="{{inputs.parameters."
+                                           "dflow_artifact_key}}")
+                        template.outputs.artifacts[art.name].save.append(
+                            S3Artifact(
+                                key="{{inputs.parameters."
+                                "dflow_artifact_key}}/%s" % name))
+
+                    if new_template.outputs.artifacts[name]._from is not \
+                            None:
+                        merge_output_artifact(
+                            new_template.outputs.artifacts[name]._from)
+                    elif new_template.outputs.artifacts[name].\
+                            from_expression is not None:
+                        merge_output_artifact(new_template.outputs.artifacts[
+                            name].from_expression._then)
+                        merge_output_artifact(new_template.outputs.artifacts[
+                            name].from_expression._else)
+
             if self.key is not None:
                 self.prepare_step = self.__class__(
                     name="%s-init-artifact" % self.name,
@@ -750,7 +918,9 @@ class Step:
         parameters = deepcopy(self.inputs.parameters)
         for name, par in parameters.items():
             value = par.value
-            if isinstance(value, (InputParameter, OutputParameter)):
+            if isinstance(value, FutureLen):
+                par.value = value.get()
+            elif isinstance(value, (InputParameter, OutputParameter)):
                 par.value = get_var(value, context).value
             elif isinstance(value, str):
                 par.value = render_expr(par.value, context)
@@ -851,7 +1021,7 @@ class Step:
 
         if self.with_param is not None or self.with_sequence is not None:
             if isinstance(self.with_param, FutureRange):
-                item_list = self.with_param.get()
+                item_list = self.with_param.get(context)
             elif isinstance(self.with_param, (InputParameter,
                                               OutputParameter)):
                 item_list = self.with_param.value
@@ -1055,7 +1225,6 @@ class Step:
                     par.value = render_item(par.value, item)
 
         import os
-        import shutil
         cwd = os.getcwd()
         if "dflow_key" in parameters:
             step_id = parameters["dflow_key"].value
@@ -1180,13 +1349,12 @@ class Step:
                     save_path = os.path.join(cwd, "..", key)
                     os.makedirs(os.path.dirname(save_path), exist_ok=True)
 
-                    def link(src, dst):
+                    def try_link(src, dst):
                         try:
                             os.symlink(src, dst)
                         except Exception:
                             pass
-                    shutil.copytree(path, save_path, copy_function=link,
-                                    dirs_exist_ok=True)
+                    merge_dir(path, save_path, try_link)
                     art.local_path = save_path
         self.record_output_artifacts(stepdir, self.outputs.artifacts)
 
@@ -1278,6 +1446,11 @@ def get_var(expr, context):
 def eval_bool_expr(expr):
     # For the original evaluator in argo, please refer to
     # https://github.com/antonmedv/expr
+    try:
+        return eval(expr)
+    except Exception:
+        pass
+
     expr_list = expr.split()
     operator = expr_list[1]
 
