@@ -19,7 +19,7 @@ from typing import List, Optional, Set, Tuple, Union
 import jsonpickle
 
 from .common import LocalArtifact, S3Artifact
-from .config import config
+from .config import config, s3_config
 
 try:
     from minio import Minio
@@ -27,31 +27,24 @@ try:
 except Exception:
     pass
 
-s3_config = {
-    "endpoint": "127.0.0.1:9000",
-    "console": "http://127.0.0.1:9001",
-    "access_key": "admin",
-    "secret_key": "password",
-    "secure": False,
-    "bucket_name": "my-bucket"
-}
 
-
-def set_s3_config(
-    **kwargs,
-) -> None:
-    """
-    Set S3 configurations
-
-    Args:
-        endpoint: endpoint for S3 storage
-        console: console address for S3 storage
-        access_key: access key for S3 storage
-        secret_key: secret key for S3 storage
-        secure: secure or not
-        bucket_name: name of S3 bucket
-    """
-    s3_config.update(kwargs)
+def get_key(artifact, raise_error=True):
+    if hasattr(artifact, "s3"):
+        return artifact.s3.key
+    elif hasattr(artifact, "oss"):
+        key = artifact.oss.key
+        if key.startswith(s3_config["repo_prefix"]):
+            return key[len(s3_config["repo_prefix"]):]
+        else:
+            return key
+    elif hasattr(artifact, "key"):
+        return artifact.key
+    else:
+        if raise_error:
+            raise FileNotFoundError(
+                "The artifact does not exist in the storage.")
+        else:
+            return None
 
 
 def download_artifact(
@@ -82,12 +75,7 @@ def download_artifact(
         linktree(artifact.local_path, path)
         return assemble_path_list(path, remove=True)
 
-    if hasattr(artifact, "s3"):
-        key = artifact.s3.key
-    elif hasattr(artifact, "key"):
-        key = artifact.key
-    else:
-        raise FileNotFoundError("The artifact does not exist in the storage.")
+    key = get_key(artifact)
 
     if slice is not None:
         sub_path = path_list_of_artifact(artifact)[slice]
@@ -206,19 +194,8 @@ def copy_artifact(src, dst, sort=False) -> S3Artifact:
         dst: destination artifact
         sort: append the path list of dst after that of src
     """
-    if hasattr(src, "s3"):
-        src_key = src.s3.key
-    elif hasattr(src, "key"):
-        src_key = src.key
-    else:
-        raise NotImplementedError()
-
-    if hasattr(dst, "s3"):
-        dst_key = dst.s3.key
-    elif hasattr(dst, "key"):
-        dst_key = dst.key
-    else:
-        raise NotImplementedError()
+    src_key = get_key(src)
+    dst_key = get_key(dst)
 
     ignore_catalog = False
     if sort:
@@ -254,32 +231,18 @@ def download_s3(
         key: str,
         path: os.PathLike = None,
         recursive: bool = True,
-        endpoint: str = None,
-        access_key: str = None,
-        secret_key: str = None,
-        secure: bool = None,
-        bucket_name: str = None,
         skip_exists: bool = False,
         **kwargs,
 ) -> str:
-    if endpoint is None:
-        endpoint = s3_config["endpoint"]
-    if access_key is None:
-        access_key = s3_config["access_key"]
-    if secret_key is None:
-        secret_key = s3_config["secret_key"]
-    if secure is None:
-        secure = s3_config["secure"]
-    if bucket_name is None:
-        bucket_name = s3_config["bucket_name"]
     if path is None:
         path = "."
-    client = Minio(endpoint=endpoint, access_key=access_key,
-                   secret_key=secret_key, secure=secure)
+    if s3_config["storage_client"] is not None:
+        client = s3_config["storage_client"]
+    else:
+        client = MinioClient(**kwargs)
     if recursive:
-        for obj in client.list_objects(bucket_name=bucket_name, prefix=key,
-                                       recursive=True):
-            rel_path = obj.object_name[len(key):]
+        for obj in client.list(prefix=key, recursive=True):
+            rel_path = obj[len(key):]
             if rel_path[:1] == "/":
                 rel_path = rel_path[1:]
             if rel_path == "":
@@ -288,20 +251,16 @@ def download_s3(
                 file_path = os.path.join(path, rel_path)
 
             if skip_exists and os.path.isfile(file_path):
-                remote_md5 = client.stat_object(
-                    bucket_name=bucket_name, object_name=obj.object_name).etag
+                remote_md5 = client.get_md5(key=obj)
                 local_md5 = get_md5(file_path)
                 if remote_md5 == local_md5:
-                    logging.debug("skip object: %s" % obj.object_name)
+                    logging.debug("skip object: %s" % obj)
                     continue
 
-            client.fget_object(bucket_name=bucket_name,
-                               object_name=obj.object_name,
-                               file_path=file_path)
+            client.download(key=obj, path=file_path)
     else:
         path = os.path.join(path, os.path.basename(key))
-        client.fget_object(bucket_name=bucket_name,
-                           object_name=key, file_path=path)
+        client.download(key=key, path=path)
     return path
 
 
@@ -309,40 +268,26 @@ def upload_s3(
         path: os.PathLike,
         key: str = None,
         prefix: str = None,
-        endpoint: str = None,
-        access_key: str = None,
-        secret_key: str = None,
-        secure: bool = None,
-        bucket_name: str = None,
         **kwargs,
 ) -> str:
-    if endpoint is None:
-        endpoint = s3_config["endpoint"]
-    if access_key is None:
-        access_key = s3_config["access_key"]
-    if secret_key is None:
-        secret_key = s3_config["secret_key"]
-    if secure is None:
-        secure = s3_config["secure"]
-    if bucket_name is None:
-        bucket_name = s3_config["bucket_name"]
-    client = Minio(endpoint=endpoint, access_key=access_key,
-                   secret_key=secret_key, secure=secure)
+    if s3_config["storage_client"] is not None:
+        client = s3_config["storage_client"]
+    else:
+        client = MinioClient(**kwargs)
     if key is not None:
         pass
     elif prefix is not None:
         if prefix[-1] != "/":
             prefix += "/"
-        objs = list(client.list_objects(
-            bucket_name=bucket_name, prefix=prefix))
-        if len(objs) == 1 and objs[0].object_name[-1] == "/":
-            prefix = objs[0].object_name
+        objs = client.list(prefix=prefix)
+        if len(objs) == 1 and objs[0][-1] == "/":
+            prefix = objs[0]
         key = "%s%s" % (prefix, os.path.basename(path))
     else:
-        key = "upload/%s/%s" % (uuid.uuid4(), os.path.basename(path))
+        key = "%supload/%s/%s" % (s3_config["prefix"],
+                                  uuid.uuid4(), os.path.basename(path))
     if os.path.isfile(path):
-        client.fput_object(bucket_name=bucket_name,
-                           object_name=key, file_path=path)
+        client.upload(key=key, path=path)
     elif os.path.isdir(path):
         for dn, ds, fs in os.walk(path, followlinks=True):
             rel_path = dn[len(path):]
@@ -351,10 +296,8 @@ def upload_s3(
             elif rel_path[0] != "/":
                 rel_path = "/" + rel_path
             for f in fs:
-                client.fput_object(bucket_name=bucket_name,
-                                   object_name="%s%s/%s" %
-                                   (key, rel_path, f),
-                                   file_path=os.path.join(dn, f))
+                client.upload(key="%s%s/%s" % (key, rel_path, f),
+                              path=os.path.join(dn, f))
     return key
 
 
@@ -362,86 +305,55 @@ def copy_s3(
         src_key: str,
         dst_key: str,
         recursive: bool = True,
-        endpoint: str = None,
-        access_key: str = None,
-        secret_key: str = None,
-        secure: bool = None,
-        bucket_name: str = None,
         ignore_catalog: bool = False,
         **kwargs,
 ) -> None:
-    if endpoint is None:
-        endpoint = s3_config["endpoint"]
-    if access_key is None:
-        access_key = s3_config["access_key"]
-    if secret_key is None:
-        secret_key = s3_config["secret_key"]
-    if secure is None:
-        secure = s3_config["secure"]
-    if bucket_name is None:
-        bucket_name = s3_config["bucket_name"]
-    client = Minio(endpoint=endpoint, access_key=access_key,
-                   secret_key=secret_key, secure=secure)
+    if s3_config["storage_client"] is not None:
+        client = s3_config["storage_client"]
+    else:
+        client = MinioClient(**kwargs)
     if recursive:
         if src_key[-1] != "/":
             src_key += "/"
-        src_objs = list(client.list_objects(
-            bucket_name=bucket_name, prefix=src_key))
-        if len(src_objs) == 1 and src_objs[0].object_name[-1] == "/":
-            src_key = src_objs[0].object_name
+        src_objs = client.list(prefix=src_key)
+        if len(src_objs) == 1 and src_objs[0][-1] == "/":
+            src_key = src_objs[0]
         if dst_key[-1] != "/":
             dst_key += "/"
-        dst_objs = list(client.list_objects(
-            bucket_name=bucket_name, prefix=dst_key))
-        if len(dst_objs) == 1 and dst_objs[0].object_name[-1] == "/":
-            dst_key = dst_objs[0].object_name
-        for obj in client.list_objects(bucket_name=bucket_name,
-                                       prefix=src_key, recursive=True):
+        dst_objs = client.list(prefix=dst_key)
+        if len(dst_objs) == 1 and dst_objs[0][-1] == "/":
+            dst_key = dst_objs[0]
+        for obj in client.list(prefix=src_key, recursive=True):
             if ignore_catalog:
-                fields = obj.object_name.split("/")
+                fields = obj.split("/")
                 if len(fields) > 1 and fields[-2] == \
                         config["catalog_dir_name"]:
                     continue
-            client.copy_object(bucket_name, dst_key + obj.object_name[len(
-                src_key):], CopySource(bucket_name, obj.object_name))
+            client.copy(obj, dst_key + obj[len(src_key):])
     else:
-        client.copy_object(bucket_name, dst_key,
-                           CopySource(bucket_name, src_key))
+        client.copy(src_key, dst_key)
 
 
 def catalog_of_artifact(art, **kwargs) -> List[dict]:
-    if hasattr(art, "s3"):
-        key = art.s3.key
-    elif hasattr(art, "key"):
-        key = art.key
-    else:
+    key = get_key(art, raise_error=False)
+    if not key:
         return []
     if key[-1] != "/":
         key += "/"
 
-    endpoint = kwargs["endpoint"] if "endpoint" in kwargs \
-        else s3_config["endpoint"]
-    access_key = kwargs["access_key"] if "access_key" in kwargs \
-        else s3_config["access_key"]
-    secret_key = kwargs["secret_key"] if "secret_key" in kwargs \
-        else s3_config["secret_key"]
-    secure = kwargs["secure"] if "secure" in kwargs else s3_config["secure"]
-    bucket_name = kwargs["bucket_name"] if "bucket_name" in kwargs \
-        else s3_config["bucket_name"]
-
-    client = Minio(endpoint=endpoint, access_key=access_key,
-                   secret_key=secret_key, secure=secure)
+    if s3_config["storage_client"] is not None:
+        client = s3_config["storage_client"]
+    else:
+        client = MinioClient(**kwargs)
     catalog = []
     with tempfile.TemporaryDirectory() as tmpdir:
-        objs = list(client.list_objects(bucket_name=bucket_name, prefix=key))
-        if len(objs) == 1 and objs[0].object_name[-1] == "/":
-            key = objs[0].object_name
+        objs = client.list(prefix=key)
+        if len(objs) == 1 and objs[0][-1] == "/":
+            key = objs[0]
         prefix = key + config["catalog_dir_name"] + "/"
-        for obj in client.list_objects(bucket_name=bucket_name, prefix=prefix):
-            fname = obj.object_name[len(prefix):]
-            client.fget_object(
-                bucket_name=bucket_name, object_name=obj.object_name,
-                file_path=os.path.join(tmpdir, fname))
+        for obj in client.list(prefix=prefix):
+            fname = obj[len(prefix):]
+            client.download(key=obj, path=os.path.join(tmpdir, fname))
             with open(os.path.join(tmpdir, fname), "r") as f:
                 catalog += jsonpickle.loads(f.read())['path_list']
     return catalog
@@ -451,21 +363,27 @@ def path_list_of_artifact(art, **kwargs) -> List[str]:
     return convert_dflow_list(catalog_of_artifact(art, **kwargs))
 
 
-def merge_dir(src, dst):
+def force_move(src, dst):
+    if os.path.exists(dst):
+        if os.path.samefile(src, dst):
+            return
+        os.remove(dst)
+    shutil.move(src, dst)
+
+
+def merge_dir(src, dst, func=force_move):
     for f in os.listdir(src):
         src_file = os.path.join(src, f)
         dst_file = os.path.join(dst, f)
-        if not os.path.exists(dst_file):
-            shutil.move(src_file, dst_file)
-        elif os.path.isdir(dst_file):
-            if os.path.isdir(src_file):
-                merge_dir(src_file, dst_file)
-            else:
+        if os.path.isdir(src_file):
+            if os.path.isfile(dst_file):
+                os.remove(dst_file)
+            os.makedirs(dst_file, exist_ok=True)
+            merge_dir(src_file, dst_file, func)
+        elif os.path.isfile(src_file):
+            if os.path.isdir(dst_file):
                 shutil.rmtree(dst_file)
-                shutil.move(src_file, dst_file)
-        else:
-            os.remove(dst_file)
-            shutil.move(src_file, dst_file)
+            func(src_file, dst_file)
 
 
 def copy_file(src, dst, func=os.link):
@@ -628,13 +546,71 @@ def find_subclass(module, cls):
 
 
 def linktree(src, dst, func=os.symlink):
-    shutil.copytree(src, dst, copy_function=partial(link, func=func),
-                    dirs_exist_ok=True)
+    merge_dir(src, dst, partial(force_link, func=func))
 
 
-def link(src, dst, func=os.symlink):
+def force_link(src, dst, func=os.symlink):
     if os.path.exists(dst):
         if os.path.samefile(src, dst):
             return
         os.remove(dst)
     func(src, dst)
+
+
+class StorageClient:
+    def upload(self, key: str, path: str) -> None:
+        pass
+
+    def download(self, key: str, path: str) -> None:
+        pass
+
+    def list(self, prefix: str, recursive: bool = False) -> List[str]:
+        pass
+
+    def copy(self, src: str, dst: str) -> None:
+        pass
+
+    def get_md5(self, key: str) -> str:
+        pass
+
+
+class MinioClient(StorageClient):
+    def __init__(self,
+                 endpoint: str = None,
+                 access_key: str = None,
+                 secret_key: str = None,
+                 secure: bool = None,
+                 bucket_name: str = None,
+                 **kwargs,
+                 ) -> None:
+        self.client = Minio(
+            endpoint=endpoint if endpoint is not None else
+            s3_config["endpoint"],
+            access_key=access_key if access_key is not None else
+            s3_config["access_key"],
+            secret_key=secret_key if secret_key is not None else
+            s3_config["secret_key"],
+            secure=secure if secure is not None else s3_config["secure"],
+        )
+        self.bucket_name = bucket_name if bucket_name is not None else \
+            s3_config["bucket_name"]
+
+    def upload(self, key: str, path: str) -> None:
+        self.client.fput_object(bucket_name=self.bucket_name,
+                                object_name=key, file_path=path)
+
+    def download(self, key: str, path: str) -> None:
+        self.client.fget_object(bucket_name=self.bucket_name,
+                                object_name=key, file_path=path)
+
+    def list(self, prefix: str, recursive: bool = False) -> List[str]:
+        return [obj.object_name for obj in self.client.list_objects(
+            bucket_name=self.bucket_name, prefix=prefix, recursive=recursive)]
+
+    def copy(self, src: str, dst: str) -> None:
+        self.client.copy_object(self.bucket_name, dst,
+                                CopySource(self.bucket_name, src))
+
+    def get_md5(self, key: str) -> str:
+        return self.client.stat_object(bucket_name=self.bucket_name,
+                                       object_name=key).etag
