@@ -6,22 +6,23 @@ from typing import Dict, List, Union
 import jsonpickle
 
 from .argo_objects import ArgoStep, ArgoWorkflow
-from .common import S3Artifact
-from .config import config
+from .config import config, s3_config
 from .context import Context
 from .context_syntax import GLOBAL_CONTEXT
 from .dag import DAG
 from .step import Step
 from .steps import Steps
 from .task import Task
-from .utils import copy_s3, linktree, randstr
+from .utils import copy_s3, get_key, linktree, randstr
 
 try:
     import kubernetes
     import urllib3
     urllib3.disable_warnings()
-    from argo.workflows.client import (ApiClient, Configuration, V1alpha1PodGC,
-                                       V1alpha1Workflow,
+    import yaml
+    from argo.workflows.client import (ApiClient, Configuration,
+                                       V1alpha1ArtifactRepositoryRef,
+                                       V1alpha1PodGC, V1alpha1Workflow,
                                        V1alpha1WorkflowCreateRequest,
                                        V1alpha1WorkflowSpec,
                                        V1LocalObjectReference, V1ObjectMeta,
@@ -64,6 +65,8 @@ class Workflow:
             * OnPodSuccess - delete pods immediately when pod is successful
             * OnWorkflowCompletion - delete pods when workflow is completed
             * OnWorkflowSuccess - delete pods when workflow is successful
+        image_pull_secrets: secrets for image registies
+        artifact_repo_key: use artifact repository reference by key
     """
 
     def __init__(
@@ -71,7 +74,7 @@ class Workflow:
             name: str = "workflow",
             steps: Steps = None,
             dag: DAG = None,
-            namespace: str = "argo",
+            namespace: str = None,
             id: str = None,
             uid: str = None,
             host: str = None,
@@ -83,6 +86,7 @@ class Workflow:
             parallelism: int = None,
             pod_gc_strategy: str = None,
             image_pull_secrets: Union[str, List[str]] = None,
+            artifact_repo_key: str = None,
     ) -> None:
         self.host = host if host is not None else config["host"]
         self.token = token if token is not None else config["token"]
@@ -99,6 +103,8 @@ class Workflow:
         if isinstance(image_pull_secrets, str):
             image_pull_secrets = [image_pull_secrets]
         self.image_pull_secrets = image_pull_secrets
+        self.artifact_repo_key = artifact_repo_key if artifact_repo_key is \
+            not None else s3_config["repo_key"]
 
         configuration = Configuration(host=self.host)
         configuration.verify_ssl = False
@@ -111,7 +117,8 @@ class Workflow:
 
         self.api_instance = WorkflowServiceApi(api_client)
 
-        self.namespace = namespace
+        self.namespace = namespace if namespace is not None else \
+            config["namespace"]
         self.id = id
         # alias uid to id if uid not provided
         if uid is None:
@@ -130,6 +137,39 @@ class Workflow:
         self.argo_templates = {}
         self.pvcs = {}
 
+        if self.k8s_api_server is not None:
+            k8s_configuration = kubernetes.client.Configuration(
+                host=self.k8s_api_server)
+            k8s_configuration.verify_ssl = False
+            if self.token is None:
+                k8s_client = kubernetes.client.ApiClient(
+                    k8s_configuration)
+            else:
+                k8s_client = kubernetes.client.ApiClient(
+                    k8s_configuration, header_name='Authorization',
+                    header_value='Bearer %s' % self.token)
+            self.core_v1_api = kubernetes.client.CoreV1Api(k8s_client)
+        else:
+            kubernetes.config.load_kube_config(
+                config_file=self.k8s_config_file)
+            self.core_v1_api = kubernetes.client.CoreV1Api()
+
+        if self.artifact_repo_key is not None:
+            cm = self.core_v1_api.read_namespaced_config_map(
+                namespace=self.namespace, name="artifact-repositories")
+            repo = yaml.full_load(cm.data[self.artifact_repo_key])
+            s3_config["repo"] = repo
+            if "s3" in repo:
+                s3_config["repo_type"] = "s3"
+                s3 = repo["s3"]
+            elif "oss" in repo:
+                s3_config["repo_type"] = "oss"
+                s3 = repo["oss"]
+            if "keyFormat" in s3:
+                t = "{{workflow.name}}/{{pod.name}}"
+                if s3["keyFormat"].endswith(t):
+                    s3_config["repo_prefix"] = s3["keyFormat"][:-len(t)]
+
     def __enter__(self) -> 'Workflow':
         GLOBAL_CONTEXT.in_context = True
         GLOBAL_CONTEXT.current_workflow = self
@@ -300,19 +340,16 @@ class Workflow:
                         for name, art in step.outputs.artifacts.items():
                             if hasattr(step, "inputs") and \
                                 hasattr(step.inputs, "parameters") and \
-                                "dflow_group_key" in \
-                                    step.inputs.parameters:
-                                if hasattr(art, "s3") and \
-                                        art.s3.key not in copied_keys:
-                                    key = "%s/%s/%s" % (
-                                        self.id,
+                                "dflow_group_key" in step.inputs.parameters \
+                                    and name != "main-logs":
+                                old_key = get_key(art, raise_error=False)
+                                if old_key and old_key not in copied_keys:
+                                    key = "%s%s/%s/%s" % (
+                                        s3_config["prefix"], self.id,
                                         step.inputs.parameters[
                                             "dflow_group_key"].value, name)
-                                    copy_s3(art.s3.key, key)
-                                    copied_keys.append(art.s3.key)
-                            if hasattr(art, "s3") and isinstance(art.s3,
-                                                                 S3Artifact):
-                                art.s3 = art.s3.to_dict()
+                                    copy_s3(old_key, key)
+                                    copied_keys.append(old_key)
                         outputs["artifacts"] = [
                             art.recover()
                             for art in step.outputs.artifacts.values()]
@@ -325,24 +362,8 @@ class Workflow:
                 config_map = kubernetes.client.V1ConfigMap(
                     data=data, metadata=kubernetes.client.V1ObjectMeta(
                         name="dflow-%s-%s" % (self.id, step.key)))
-                if self.k8s_api_server is not None:
-                    k8s_configuration = kubernetes.client.Configuration(
-                        host=self.k8s_api_server)
-                    k8s_configuration.verify_ssl = False
-                    if self.token is None:
-                        k8s_client = kubernetes.client.ApiClient(
-                            k8s_configuration)
-                    else:
-                        k8s_client = kubernetes.client.ApiClient(
-                            k8s_configuration, header_name='Authorization',
-                            header_value='Bearer %s' % self.token)
-                    v1 = kubernetes.client.CoreV1Api(k8s_client)
-                else:
-                    kubernetes.config.load_kube_config(
-                        config_file=self.k8s_config_file)
-                    v1 = kubernetes.client.CoreV1Api()
-                v1.create_namespaced_config_map(namespace=self.namespace,
-                                                body=config_map)
+                self.core_v1_api.create_namespaced_config_map(
+                    namespace=self.namespace, body=config_map)
             self.handle_template(
                 self.entrypoint, memoize_prefix=self.id,
                 memoize_configmap="dflow")
@@ -380,7 +401,9 @@ class Workflow:
                 volume_claim_templates=argo_pvcs,
                 pod_gc=V1alpha1PodGC(strategy=self.pod_gc_strategy),
                 image_pull_secrets=None if self.image_pull_secrets is None else
-                [V1LocalObjectReference(s) for s in self.image_pull_secrets]
+                [V1LocalObjectReference(s) for s in self.image_pull_secrets],
+                artifact_repository_ref=None if self.artifact_repo_key is None
+                else V1alpha1ArtifactRepositoryRef(key=self.artifact_repo_key)
             ),
             status=status)
 
